@@ -55,21 +55,22 @@ class HyprlandWorkspaces(Gtk.Box):
             self.pack_start(self.num_box, False, False, 0)
 
             # read workspaces from workspace rules
-            ws_rules = h_list_workspace_rules()
-            workspace_from_rules = []
+            ws_rules = [rule for rule in h_list_workspace_rules() if self.is_workspace_rule_valid(rule)]
+            workspace_rules = []
             for ws in ws_rules:
                 if self.settings["show-workspaces-from-all-outputs"] or ws["monitor"] == self.monitor_name:
-                    workspace_from_rules.append(ws)
-            self.workspace_from_rules = workspace_from_rules  # storing the workspaces for the current monitor from rules
+                    workspace_rules.append(ws)
+            self.workspace_rules = workspace_rules  # storing the workspace rules for the current monitor from rules
 
             if self.settings["show-inactive-workspaces"]:
-                self.ws_nums = [int(ws["workspaceString"]) for ws in workspace_from_rules]
+                self.ws_nums = [int(ws["workspaceString"]) for ws in workspace_rules]
             else:
                 self.ws_nums = []
             # Creating a list of workspaces from active workspaces
-            for ws in workspaces[:self.settings["num-ws"]]:
+            for ws in workspaces:
                 if (
                         (self.settings["show-workspaces-from-all-outputs"] or ws["monitor"] == self.monitor_name)
+                        and ws["id"] > 0 # filter special workspaces
                         and ws["id"] not in self.ws_nums
                 ):
                     self.ws_nums.append(ws["id"])
@@ -120,6 +121,18 @@ class HyprlandWorkspaces(Gtk.Box):
         box.pack_start(lbl, False, False, 6)
 
         return eb, lbl
+    
+    def is_workspace_rule_valid(self, ws_rule):
+        """Check if the workspace rule is defining a workspace and binding it to a monitor. If not, return False.
+        This is specific to identify nwg-panel workspace rules.
+        """
+        try:
+            int(ws_rule["workspaceString"]) # since we are only supporting workspace ids, and not workspaces defined by names
+            if "monitor" in ws_rule:
+                return True
+            return False
+        except:
+            return False
 
     def choose_workspace_ids_around_active(self, workspace_ids, active_ws_id):
         # choose workspaces around the active workspace
@@ -144,22 +157,24 @@ class HyprlandWorkspaces(Gtk.Box):
         ]
         # sort workspaces
         workspaces.sort(key=lambda x: x["id"])
-        current_mon = [m for m in monitors if m["name"] == self.monitor_name][0]
-        # active workspace on the current monitor is what we want
-        active_ws = [ws for ws in workspaces if current_mon['activeWorkspace']["id"] == ws["id"]][0]
+        # find focused monitor, return safe dict if nothing found
+        focused_mon = next((m for m in monitors if m["focused"]), {"activeWorkspace": {"id": 0}})
+        # active workspace on the focused monitor is what we want
+        active_ws = [ws for ws in workspaces if focused_mon['activeWorkspace']["id"] == ws["id"]][0]
 
         if self.settings["show-workspaces"]:
             occupied_workspaces = []  # should not be sorted, as this should be in the same order as the workspaces in Hyprland
             self.ws_id2name = {}
             if self.settings["show-inactive-workspaces"]:
                 self.ws_nums = [int(ws["workspaceString"]) for ws in
-                                self.workspace_from_rules]  # start with workspaces from rules
+                                self.workspace_rules]  # start with workspaces from rules
             else:
                 self.ws_nums = []
             # Updating occupied workspaces
             for ws in workspaces:
                 # add workspaces to the list if not already there, important for example when monitor is unplugged
-                if ws["id"] not in self.ws_nums:
+                if (ws["id"] > 0 # filter special workspaces
+                    and ws["id"] not in self.ws_nums):
                     self.ws_nums.append(ws["id"])
 
                 for client in clients:  # check for all occupied workspaces
